'use strict';

const path = require('path');
const generators = require('yeoman-generator');
const extend = require('deep-extend');

const required = (str) => str.length > 0;

module.exports = generators.Base.extend({
  constructor: function () {
    generators.Base.apply(this, arguments);
  },

  initializing: function () {
    this.props = {};
  },

  prompting: {
    gitRepo: function () {
      var prompt = [{
        name: 'gitRepo',
        message: 'Your git repository name',
        default: path.basename(process.cwd()),
        validate: required
      }];

      return this.prompt(prompt).then(function (response) {
        this.props.gitRepo = response.gitRepo;
      }.bind(this));
    },
    serviceName: function () {
      var prompt = [{
              name: 'serviceName',
              message: 'Your service name',
              default: path.basename(process.cwd()),
              validate: required
      }];

      return this.prompt(prompt).then(function (response) {
        this.props.serviceName = response.serviceName;
      }.bind(this));
    },
    handlerName: function () {
      var prompt = [{
        name :'handlerName',
        message: 'Your handler name',
        default: path.basename(process.cwd()),
        validate: required
      }]
      return this.prompt(prompt).then(function(response){
        this.props.handlerName = response.handlerName;
      }.bind(this));
    },
    eventType: function () {
      var prompt = [{
        type: 'list',
        name: 'eventType',
        message: 'Select a event type:',
        choices: [
          'http',
        ],
        store: true
      }];

      return this.prompt(prompt).then(function (response) {
        this.props.eventType = response.eventType;
      }.bind(this));
    },
    eventsHTTPPath: function () {
      var prompt = [{
        name :'eventsHTTPPath',
        message: 'Your path for your http event',
        default: path.basename(process.cwd()),
        validate: required
      }];

      return this.prompt(prompt).then(function (response) {
        this.props.eventsHTTPPath = response.eventsHTTPPath;
      }.bind(this));
    },
    eventsHTTPMethod: function () {

      var prompt = [{
        type: 'list',
        name: 'eventsHTTPMethod',
        message: 'Select method for http event:',
        choices: [
          'ANY',
          'DELETE',
          'GET',
          'HEAD',
          'OPTIONS',
          'PATCH',
          'POST',
          'PUT',
        ],
        store: true
      }];

      return this.prompt(prompt).then(function (response) {
        this.props.eventsHTTPMethod = response.eventsHTTPMethod;
      }.bind(this));
    },
  },

  writing: function () {
    const files = {
      'src/sampleLib.js': 'src/sampleLib.js',
      'test/**': 'test',
      'scripts/**': 'scripts',
      'event/**': 'event',
      'babelrc': '.babelrc',
      'eslintrc.js': '.eslintrc.js',
      'gitignore': '.gitignore',
      'circle.yml': 'circle.yml',
      'jsdoc.json': 'jsdoc.json',
      'swaggerJsdoc.config.js': 'swaggerJsdoc.config.js',
      'webpack.config.babel.js': 'webpack.config.babel.js',
      'editorconfig': '.editorconfig',
      'docker-compose.yml': 'docker-compose.yml',
      'docker-compose-test.yml': 'docker-compose-test.yml',
      'test/unit/sampleLib.test.js': 'test/unit/sampleLib.test.js'
    };

    const templates = {
      'README.md': '',
      'serverless.yml': '',
      'package.json': 'package.json',
      'src/handler.js': 'src/handler.js',
      'test/functional/handler.test.js': 'test/functional/handler.test.js'
    };

    const plainCopy = (path, dest) => this.fs.copy(path, dest);
    const templateCopy = (path, dest) => this.fs.copyTpl(path, dest, this.props);
    const iterateFiles = ((files, action) => {
      Object.keys(files).forEach((function(path) {
        const dest = files[path] || path;
        action(this.templatePath(path), this.destinationPath(dest));
      }).bind(this));
    }).bind(this);

    iterateFiles(files, plainCopy);
    iterateFiles(templates, templateCopy);
  },


});
