# lambda-boilerplate

### 1. Enable Circle CI

- Follow github project using [@plnt9machine](https://github.com/orgs/Plnt9/people/plnt9machine) account
- Login to CircleCI using [@plnt9machine](https://github.com/orgs/Plnt9/people/plnt9machine) account and follow the project
- Add `NPM_TOKEN` env variable to the CircleCI:
  - get the token by running `cat ~/.npmrc` and copy the value of `_authToken`
  - Open CircleCI console, click on a project -> settings -> environment variables
  - add variable with the name `NPM_TOKEN` and value copied from `_authToken`


### 2. Extra

If you want to add a visual **build status** badge to your repo README you can use the following snippet:

```markdown
[![Build status](https://circleci.com/gh/Plnt9/sls-name-of-your-lambda/tree/master.svg?style=shield&circle-token=<YOUR CIRCLE CI PROJECT TOKEN HERE>)](https://circleci.com/gh/Plnt9/sls-name-of-your-lambda)
```

Be sure to change the URLs with the real name of your project and to provide a valid **Project Level** token.

Also if you want to enable notification on our slack channel from Circle CI builds, go to your project settings and select `Notifications > chat notifications`. Under slack add the following callback url:

```
https://hooks.slack.com/services/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
```


## Commands available

This boilerplate is ready to provide you some commands:

  - `npm test`: run the linting and the unit tests for the current project
  - `npm run test:lint`: run only the linting
  - `npm run test:unit`: run only the unit tests
  - `npm run build`: builds the current project into a self-deployable lambda package

## Developing a Lambda

To test your lambda from your local machine you can execute the following from
the build folder:
```
sls invoke local --function $handlerName --stage $stage --path ../event/$event.json
```
The `$event.json` file is used to mock the `event` argument that is passed in to your
handler.

## What else?

Nothing really. Feel free to [bootstrap a new discussion](/issues) in case you think there's something missing here.


### Async Await functions

The boilerplate has async/await javascript functionality so the async code can be simplified
More docs on the subject: 
https://ponyfoo.com/articles/understanding-javascript-async-await 
https://www.twilio.com/blog/2015/10/asyncawait-the-hero-javascript-deserved.html 
Example

```
function getAsyncData(param1, ...){
  //returns a Promise
}

async AsyncAwaitFunction function(param1, ...) {
  const asyncResult = await getAsyncData(param1, ...)

  console.log(asyncResult)
}

```

### Functional testing

The boilerplate has docker-compose which enables functional testsing. To lift docker containers, execute ```docker-compose up```. If image is in the AWS private registry, run ```$(aws ecr get-login --region eu-west-1 --profile dev)``` to login. This will run docker container in the current shell (you can run it in daemon mode with -d flag). For listing all running containers, open new shell and execute

```bash
docker ps
```

Find container ID of the node container and execute

```bash
docker exec -it ${CONTAINER_ID} bash
```

you are now inside node docker container. you can run npm test for tests or execute any code you would normally do. All enviromental variables are setup

If you want to lift containers and execute tests automatically with 1 command, execute ```npm run test:local```
