#!/usr/bin/env node
/* eslint-disable */

const yaml = require('js-yaml');
const _ = require('lodash');
const debug = require('debug')('subscribeToLoggly');

debug('starting subscribeToLoggly script');

const stdoutWrite = process.stdout.write.bind(process.stdout);
const chunks = [];

process.stdin.on('data', chunk => chunks.push(chunk));

process.stdin.on('end', () =>
  _.flow(
    Buffer.concat,
    yaml.safeLoad,
    appendSubscriptionFilterRes,
    yaml.safeDump,
    stdoutWrite
  )(chunks));

/**
 * Append Subcription filter Resources for subscribing to loggly lambda
 * @param {Object} yaml js representation of serverless.yaml file
 * @return {Object} same yml file with appended Subscription resources
 */
const appendSubscriptionFilterRes = (_yaml) => {
  const yaml = _.cloneDeep(_yaml);
  if (!yaml.resources) {
    yaml.resources = { Resources: {} };
  }
  const Resources = yaml.resources.Resources;
  const functions = yaml.functions;
  Object.keys(functions).forEach((fun) => {
    const functionFullName = `\${self:service}-\${opt:stage}-${fun}`;
    const resourceName = `cw2loggly${fun}V${yaml.custom.version}`.replace(/[^0-9a-z]/gi, '');
    functions[fun].name = functionFullName;
    Resources[resourceName] = {
      Type: "AWS::Logs::SubscriptionFilter",
      DependsOn: `${normalizeFunctionName(fun)}LogGroup`,
      Properties: {
        DestinationArn: '${self:custom.logglyFunArn}',
        FilterPattern: '[timestamp=*Z, request_id="*-*", event]',
        LogGroupName: `/aws/lambda/${functionFullName}`,
      }
    }
  });
  return yaml;
};

/**
 * function that Normalizes Function name. Used for getting CF resource name responsible for creating CW Logs
 * @param {String} name function short name
 * @return {String} normalized function name
 */
const normalizeFunctionName = name => _.upperFirst(name).replace(/-/g, 'Dash').replace(/_/g, 'Underscore');
